// Demonstrate Simple Texturing
// Based on checker.c from the OpenGL RedBook, first edition.
// Ported to C++ and modified for Core Profile by Alex Clarke.

#include "Angel.h"
#include <iostream>
using namespace std;

//Define some geometry and colours here
GLfloat points[] = 
{
	//Square
   -2.0, -1.0, 0.0,
   -2.0, 1.0, 0.0,
   0.0, 1.0, 0.0,

   -2.0, -1.0, 0.0,
   0.0, 1.0, 0.0,
   0.0, -1.0, 0.0,

   //Triangle
   0.0, -1.0, 0.0,
   2.0, -1.0, 0.0,
   1.0,  1.0, 0.0

};

GLfloat texCoords[] = 
{
	//Square
   0.0, 0.0,
   0.0, 1.0,
   1.0, 1.0,

   0.0, 0.0,
   1.0, 1.0,
   1.0, 0.0,

   //Triangle
   0.0, 0.0,
   1.0, 0.0,
   0.0, 1.0
};


//Vertex array objects
GLuint vao;

//Shader Programs
GLuint program1;


/*	Create checkerboard texture	*/
#define	checkImageWidth 64
#define	checkImageHeight 64
static GLubyte checkImage[checkImageHeight][checkImageWidth][4];


static GLuint texName;

mat4 p;
mat4 mv;
GLuint projIndex;
GLuint mvIndex;

void makeCheckImage(void)
{
   int i, j, c;
    
   for (i = 0; i < checkImageHeight; i++) 
   {
      for (j = 0; j < checkImageWidth; j++) 
      {
	 c = ( ( ((i&0x8)==0) ^ ((j&0x8)==0) ) )*255;
         checkImage[i][j][0] = (GLubyte) c;
         checkImage[i][j][1] = (GLubyte) c;
         checkImage[i][j][2] = (GLubyte) c;
         checkImage[i][j][3] = (GLubyte) 255;
      }
   }
}


//----------------------------------------------------------------------------

// OpenGL initialization
void init()
{	
	   glClearColor(0.5, 0.5, 0.5, 1.0);

	//Set up shaders and VAOs
	program1 = InitShader( "vShaderL5-1.glsl", "fShaderL5-1.glsl" );


	glUseProgram( program1 );
	glGenVertexArrays( 1, &vao );
	glBindVertexArray( vao );

	//Load vertex positions and texture coordinates into vertex buffer.
	GLuint buffer;
	glGenBuffers( 1, &buffer );
	glBindBuffer( GL_ARRAY_BUFFER, buffer );
	glBufferData( GL_ARRAY_BUFFER, sizeof(points)+sizeof(texCoords), NULL, GL_STATIC_DRAW );
	glBufferSubData( GL_ARRAY_BUFFER, 0, sizeof(points), points );
	glBufferSubData( GL_ARRAY_BUFFER, sizeof(points), sizeof(texCoords), texCoords );
	GLuint vPosition = glGetAttribLocation( program1, "vPosition" );
	glEnableVertexAttribArray( vPosition );
	glVertexAttribPointer( vPosition, 3, GL_FLOAT, GL_FALSE, 0, BUFFER_OFFSET(0) );
	GLuint vTexCoord = glGetAttribLocation( program1, "vTexCoord" ); 
	glEnableVertexAttribArray( vTexCoord );
	glVertexAttribPointer( vTexCoord, 2, GL_FLOAT, GL_FALSE, 0, BUFFER_OFFSET(sizeof(points)) );

	//Generate a texture in system RAM
	makeCheckImage();
   

	//Tell OpenGL that textures are packed tightly in memory
	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);

	//Make texture unit 0 active
	glActiveTexture(GL_TEXTURE0);

	//Generate a texture name
   glGenTextures(1, &texName);

   //Bind the texture as current
   glBindTexture(GL_TEXTURE_2D, texName);

   //Set texture parameters
   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);

   //Load texture data into OpenGL from system RAM
   glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, checkImageWidth, checkImageHeight, 
                0, GL_RGBA, GL_UNSIGNED_BYTE, checkImage);

   //Tell shader which texture unit to use
   glUniform1i(glGetUniformLocation(program1, "tex"), 0);

   //Get transformation matrix locations from shader
   projIndex = glGetUniformLocation(program1, "p");
   mvIndex = glGetUniformLocation(program1, "mv");
   
}

//----------------------------------------------------------------------------

void display( void )
{
	//Clear the drawing buffer
	glClear( GL_COLOR_BUFFER_BIT  );

	//Draw triangle
	glDrawArrays(GL_TRIANGLES, 0, 9);

	glutPostRedisplay();
	//Show what was drawn
	glFinish();
	glutSwapBuffers();
}

//----------------------------------------------------------------------------

void reshape (int w, int h)
{
	glViewport(0, 0, (GLsizei) w, (GLsizei) h);
	p = Perspective(60.0, (GLfloat) w/(GLfloat) h, 1.0, 30.0);
	glUniformMatrix4fv(projIndex, 1, GL_TRUE, p);
	mv = Translate(0.0, 0.0, -3.6);
	glUniformMatrix4fv(mvIndex, 1, GL_TRUE, mv);
}


void keyboard( unsigned char key, int x, int y )
{
	switch( key ) {
	case 033:  // Escape key
	case 'q': case 'Q':
		exit( EXIT_SUCCESS );
		break;
	}
}

//----------------------------------------------------------------------------


int main( int argc, char **argv )
{
	glutInit( &argc, argv );
	glutInitWindowSize( 512, 512 );
    
#ifdef __APPLE__
	glutInitDisplayMode( GLUT_3_2_CORE_PROFILE | GLUT_RGBA | GLUT_DOUBLE | GLUT_DEPTH );
#else
	// If you are using freeglut, the next two lines will check if
	// the code is truly 3.2. Otherwise, comment them out
	glutInitContextVersion( 3, 2 );
	glutInitContextProfile( GLUT_CORE_PROFILE );
	glutInitDisplayMode( GLUT_RGBA | GLUT_DOUBLE | GLUT_DEPTH );
    
#endif
    
    glutCreateWindow( "Lab 5 - Checkers" );
    glewExperimental = GL_TRUE;
    glewInit();
    
	init();
    
	glutDisplayFunc( display );
	glutKeyboardFunc( keyboard );
	glutReshapeFunc( reshape );
	glutMainLoop();

	return 0;
}
